@@ -1,8 +1,10 @@
 #!/usr/bin/env python
-"""app.engine.cma_sleepwake: define the Cortisol-Melatonin-Adiponectin model.
-"""
+"""app.engine.cma: define the Cortisol-Melatonin-Adiponectin model."""
+from pfun_cma_model.misc.types import NumpyArray
+from pfun_cma_model.engine.cma_model_params import CMAModelParams, CMABoundedParams
+from pfun_cma_model.engine.bounds import Bounds
+from pfun_cma_model.engine.calc import exp, Light, E, vectorized_G
 from functools import cached_property
-import importlib
 import copy
 import json
 import logging
@@ -10,8 +12,14 @@ import sys
 from pathlib import Path
 from typing import (
     Sequence,
-    Callable, Container, Dict, Iterable, Tuple,
-    Optional, Generator
+    Iterable,
+    Dict,
+    Tuple,
+    Callable,
+    Container,
+    Optional,
+    Generator,
+    Annotated,
 )
 from numpy import (
     array,
@@ -20,26 +28,18 @@ from numpy import (
     power,
     cos,
     pi,
-    piecewise,
     logical_and,
     logical_or,
     append,
     mod,
     linspace,
     broadcast_to,
-    atleast_1d,
-    zeros,
     bool_,
-    asarray
+    asarray,
 )
 from numpy import abs as np_abs
 from numpy.random import default_rng
-from pandas import (
-    DataFrame,
-    Series,
-    json_normalize,
-    to_timedelta
-)
+from pandas import DataFrame, Series, json_normalize, to_timedelta
 
 #: pfun imports (relative)
 root_path = str(Path(__file__).parents[1])
@@ -48,27 +48,12 @@ if root_path not in sys.path:
     sys.path.insert(0, root_path)
 if mod_path not in sys.path:
     sys.path.insert(0, mod_path)
-try:
-    from pfun_cma_model.engine.calc import (
-        exp,
-        Light,
-        E,
-        vectorized_G
-    )
-    from pfun_cma_model.engine.bounds import Bounds
-    from pfun_cma_model.engine.cma_model_params import CMAModelParams
-except ModuleNotFoundError:
-    exp = importlib.import_module(
-        ".calc", package="pfun_cma_model.engine").exp
-    Bounds = importlib.import_module(
-        ".bounds", package="pfun_cma_model.engine").Bounds
-    CMAModelParams = importlib.import_module(
-        ".cma_model_params", package="pfun_cma_model.engine").CMAModelParams
+
 
 logger = logging.getLogger()
 
 __all__ = [
-    'CMASleepWakeModel',
+    "CMASleepWakeModel",
 ]
 
 
@@ -80,19 +65,17 @@ class CMAParamTypeError(TypeError):
         self._pkey = pkey
 
     def __repr__(self) -> str:
-        return super().__repr__() + \
-            f"...Parameter '{self._pkey}' must be numeric."
+        return super().__repr__() + f"...Parameter '{self._pkey}' must be numeric."
 
 
 class CMASleepWakeModel:
-
     """Defines the Cortisol-Melatonin-Adiponectin Sleep-Wake pfun model.
 
     Methods:
     -------
-    1) Input SG -> Project SG to 24-hour phase plane.
-    2) Estimate photoperiod (t_m0 - 1, t_m2 + 3) -> Model params (d, taup).
-    3) (Fit to projected SG) Compute approximate chronometabolic dynamics:
+    1)) Input SG -> Project SG to 24-hour phase plane.
+    2)) Estimate photoperiod (t_m0 - 1, t_m2 + 3) -> Model params (d, taup).
+    3)) (Fit to projected SG) Compute approximate chronometabolic dynamics:
         F(m, c, a)(t, d, taup) -> ...
          ...  (+/- postprandial insulin, glucose){Late, Early}.
     """
@@ -107,36 +90,40 @@ class CMASleepWakeModel:
 
     @cached_property
     def bounded_param_keys(self):
-        return self._DEFAULT_PARAMS_MODEL.bounded_param_keys
+        return self._DEFAULT_PARAMS_MODEL.bounded.bounded_param_keys
 
     @cached_property
     def param_keys(self):
         return tuple(self._DEFAULT_PARAMS.keys())
 
-    def param_key_index(self,
-                        keys: str | Iterable[str] | Sequence[str],
-                        only_bounded: bool = False
-                        ) -> int | Iterable[int]:
+    def param_key_index(
+        self, keys: str | Iterable[str] | Sequence[str], only_bounded: bool = False
+    ) -> int | list[int]:
         """Return the index of the parameter key."""
-        local_param_keys = self.param_keys if not only_bounded else \
-            self.bounded_param_keys
+        local_param_keys = (
+            self.param_keys if not only_bounded else self.bounded_param_keys
+        )
         if isinstance(keys, str):
             return local_param_keys.index(keys)
         else:
-            return [
-                int(self.param_key_index(k, only_bounded=only_bounded))  # type: ignore
-                for k in keys
-            ]
+            return [local_param_keys.index(k) for k in keys]
 
-    def update_bounds(self, keys=[], lb=[], ub=[],
-                      keep_feasible: bool_ | Iterable[bool_] = Bounds.True_,
-                      return_bounds=False):
+    def update_bounds(
+        self,
+        keys=[],
+        lb=[],
+        ub=[],
+        keep_feasible: bool_ | Iterable[bool_] = Bounds.True_,
+        return_bounds=False,
+    ):
         """Update the bounds of the model."""
         keys = [keys] if isinstance(keys, str) else keys
         lb = [float(lb)] if isinstance(lb, (float, int)) else lb
         ub = [float(ub)] if isinstance(ub, (float, int)) else ub
         if isinstance(keep_feasible, bool_):
-            keep_feasible = [keep_feasible, ] * len(keys)
+            keep_feasible = [
+                keep_feasible,
+            ] * len(keys)
         for k in keys:
             ix = self.bounded_param_keys.index(k)
             self.bounds[ix] = (
@@ -159,13 +146,15 @@ class CMASleepWakeModel:
                 out[key] = value.tolist()
             elif isinstance(value, Bounds):
                 out[key] = value.json()
-            elif hasattr(value, '__json__'):
+            elif hasattr(value, "__json__"):
                 out[key] = value.__json__()
-            elif hasattr(value, 'model_dump'):
+            elif hasattr(value, "model_dump"):
                 out[key] = value.model_dump()
             elif isinstance(value, (Generator)):
                 logging.warning(
-                    "Could not convert %s (type=%s) to JSON.", str(value), type(value))
+                    "Could not convert %s (type=%s) to JSON.", str(
+                        value), type(value)
+                )
                 out.pop(key)
             elif isinstance(value, (dict)):
                 for k, v in value.items():
@@ -174,16 +163,21 @@ class CMASleepWakeModel:
                     elif isinstance(v, ndarray):
                         value[k] = v.tolist()
             try:
-                logging.info("attempting to serialize: %s (value='%s')", key, str(out.get(key)))
+                logging.info(
+                    "attempting to serialize: %s (value='%s')", key, str(
+                        out.get(key))
+                )
                 json.dumps(out[key])
             except (json.JSONDecodeError, TypeError) as exc:
                 logging.warning(
                     "Exception: '%s'\n...Could not convert %s to JSON.",
                     str(exc),
                     str(value),
-                    exc_info=False)
+                    exc_info=False,
+                )
                 out.pop(key, None)  # ! remove any non-JSONable value
-        return json.dumps(out, skipkeys=True)  # ! skipkeys removes all non-basic types
+        # ! skipkeys removes all non-basic types
+        return json.dumps(out, skipkeys=True)
 
     def json(self):
         """JSON serialization."""
@@ -196,13 +190,30 @@ class CMASleepWakeModel:
         return self.to_dict()
 
     @property
-    def params(self):
-        return self._params
+    def params(self) -> CMAModelParams:
+        """Return the current parameters as a CMAModelParams object."""
+        params_dict = {k: self._params[k]
+                       for k in self.param_keys}  # preserve order
+        return CMAModelParams(**params_dict)
 
-    @params.setter
+    # class-level private storage of parameters
+    _params: Dict = CMAModelParams().model_dump()
+
+    def new_tvector(self, t0: int | float, t1: int | float, n: int) -> ndarray:
+        return self.params.new_tvector(t0, t1, n)
+
+    def __getitem__(self, key):
+        """Get parameter value by key."""
+        return getattr(self.params, key)
+
+    @params.setter  # type: ignore
     def params(self, value):
         self._params = value
 
+    @property
+    def unbounded_param_keys(self):
+        return ("t", "N", "tM", "seed", "eps")
+
     def __init__(self, config: Dict | CMAModelParams | None = None, **kwds):  # type: ignore
         """PFun CMA model constructor.
 
@@ -220,66 +231,109 @@ class CMASleepWakeModel:
             seed (None | int, optional): Random seed value. If provided, random noise will be included in the model solution, scaled by parameter eps. Defaults to None.
             eps (float, optional): Random noise scale ("epsilon"). Defaults to 1e-18.
         """
-        self._params: Dict = {}
-        defaults = self._DEFAULT_PARAMS
-        self._params.update(defaults)  # type: ignore
+        # update with any given config:
         if config is not None:
             if isinstance(config, CMAModelParams):
-                config = config.model_dump()  # type: ignore
-            self._params.update(config)  # type: ignore
+                config = config.model_dump()
+            self._params.update(config)
+        # update with any given kwds:
         self._params.update(kwds)
-        t, N, tM, seed, eps = self._params.get('t'), self._params.get('N'), self._params.get(
-            'tM'), self._params.get('seed'), self._params.get('eps')  # type: ignore
-        assert (t is not None or N is not None) and (t is None or N is None), \
-            "Must provide either the 't' or 'N' argument (not both)"
-        if t is None:
-            t = self.new_tvector(0, 24, N)
-        self.t: Sequence[float] | ndarray = t  # time vector  # type: ignore
-        self.tM = asarray(tM, dtype=float)  # mealtimes vector
+        # Setup bounds (for bounded params):
         self.bounds = copy.copy(self._DEFAULT_PARAMS_MODEL.bounds)
-        if all([kwds.get('lb', False), kwds.get('ub', False), kwds.get('bounded_param_keys', False)]):
-            self.update_bounds(kwds['bounded_param_keys'], kwds['lb'], kwds['ub'], kwds.get('keep_feasible', Bounds.True_))
-        elif 'bounds' in kwds:
-            new_bounds = kwds['bounds']
+        if all(
+            [
+                kwds.get("lb", False),
+                kwds.get("ub", False),
+                kwds.get("bounded_param_keys", False),
+            ]
+        ):
+            self.update_bounds(
+                kwds["bounded_param_keys"],
+                kwds["lb"],
+                kwds["ub"],
+                kwds.get("keep_feasible", Bounds.True_),
+            )
+        elif "bounds" in kwds:
+            new_bounds = kwds["bounds"]
             if isinstance(new_bounds, str):
                 new_bounds = json.loads(new_bounds)
             self.update_bounds(**new_bounds)
-        self.eps = eps
+        # Setup the random number generator (if seed is given)
         self.rng = None
-        if seed is not None:
-            self.rng = default_rng(seed=seed)  # type: ignore
-
-    @staticmethod
-    def new_tvector(t0: int | float, t1: int | float, n: int) -> ndarray:
-        """Create a new linear time vector, given initial (t0), final (t1), and number of timepoints (n)"""
-        return linspace(t0, t1, num=int(n))
+        if self.seed is not None:
+            self.rng = default_rng(seed=self.seed)
 
     @property
-    def N(self):
-        """Number of time steps."""
-        return len(self.t)
-    
+    def eps(self) -> float:
+        """eps : float
+        Random noise scale ("epsilon").
+        """
+        return self.params.eps
+
+    @property
+    def seed(self) -> Optional[int | float]:
+        """seed : Optional[int | float]
+        Random seed. Set to an integer to enable random noise via parameter 'eps'. Optional.
+        """
+        return self.params.seed
+
+    @property
+    def tM(self) -> ndarray:
+        """tM : ndarray
+        Meal times (hours).
+        """
+        return array(self.params.tM)  # type: ignore
+
+    @tM.setter  # type: ignore
+    def tM(self, value):
+        self.params.tM = array(value)  # type: ignore
+
+    @property
+    def t(self) -> ndarray:
+        """t : ndarray
+        Time vector (decimal hours).
+        """
+        return array(self.params.t)
+
+    @t.setter  # type: ignore
+    def t(self, value):
+        self.params.t = array(value)  # type: ignore
+
+    @property
+    def N(self) -> int:
+        """N : int
+        Number of time points.
+
+        Default to self.t.size.
+        """
+        return int(self.params.N)
+
+    @N.setter  # type: ignore
+    def N(self, value):
+        self.params.N = int(value)
+
     @property
     def bounded_params_as_dict(self) -> Dict:
         """Return the current bounded parameters as a python dict."""
-        return {k: self.params[k] for k in self.bounded_param_keys}
-    
+        return {k: self.params.bounded[k] for k in self.bounded_param_keys}
+
     @property
-    def bounded_params(self) -> CMAModelParams:
+    def bounded_params(self) -> CMABoundedParams:
         """Return the current bounded parameters as a CMAModelParams object."""
-        return CMAModelParams(**self.bounded_params_as_dict)
-    
+        return self.params.bounded
+
     @property
-    def bounded_params_as_obj(self) -> CMAModelParams:
+    def bounded_params_as_obj(self) -> CMABoundedParams:
         """
         Return the current bounded parameters as a CMAModelParams object.
-        
+
         Alias method for `self.bounded_params` property.
         """
         return self.bounded_params
 
-    def update_bounded_params(self, params: Dict | CMAModelParams  # type: ignore
-                              ) -> Dict | CMAModelParams:  # type: ignore
+    def update_bounded_params(
+        self, params: Dict | CMAModelParams | CMABoundedParams
+    ) -> CMAModelParams:
         """Update the latest parameters to correspond to the current bounds (trim to bounds).
 
         Args:
@@ -288,12 +342,25 @@ class CMASleepWakeModel:
         Returns:
             Dict: updated parameters
         """
-        bounded_params = {k: v for k, v in params.items() if k in self.bounded_param_keys}
-        new_params = self.bounds.update_values(bounded_params)
-        params.update(new_params)
-        return params
+        # get the bounded parameters from the params dict
+        if isinstance(params, CMAModelParams):
+            params = params.model_dump()
+        # If params is already a dict, do not call model_dump
+        bounded_params = {
+            k: v for k, v in params.items() if k in self.bounded_param_keys
+        }
+        bounded_params.update(params.get("bounded", {}))  # type: ignore
+        # ! ensure the bounded parameters are within bounds
+        new_params = self.bounds.update_values(
+            {k: float(v) for k, v in bounded_params.items()}
+        )
+        self.params.bounded.update(**new_params)
+        return self.params
 
-    def update(self, model_params: CMAModelParams | Dict | None = None, inplace=True, **kwds):  # type: ignore
+    # type: ignore
+    def update(
+        self, model_params: Optional[CMAModelParams | Dict] = None, inplace=True, **kwds
+    ):
         """
         Update the current instance with new values (occurs inplace by default).
 
@@ -322,58 +389,85 @@ class CMASleepWakeModel:
         """
         if model_params is not None:
             if isinstance(model_params, CMAModelParams):
-                model_params: CMAModelParams | Dict = model_params.dict()  # type: ignore
-            model_params.update(kwds)
-            kwds = dict(model_params)
+                model_params = model_params.model_dump()  # convert to dict
+            model_params.update(**kwds)  # type: ignore
+            kwds = dict(model_params)  # type: ignore
         if inplace is False:
             new_inst = copy.copy(self)
             new_inst.update(inplace=True, **kwds)
             return new_inst
         #: ! handle case in which taug was given as a vector initially
-        if 'taug' in kwds and isinstance(self.params['taug'], Container):
-            taug_new = kwds.pop('taug')
+        if "taug" in kwds and isinstance(getattr(self.params, "taug", None), Container):
+            taug_new = kwds.pop("taug")
             match isinstance(taug_new, Container):
                 case True:
                     #: ! replace current values elementwise if given a vector
-                    self.params['taug'] = broadcast_to(  # type: ignore
-                        taug_new, (self.n_meals, ))
+                    self.params["taug"] = broadcast_to(  # type: ignore
+                        taug_new, (self.n_meals,)
+                    )
                 case False:  # ! else, taug is a scale: <old_taug> *= new_taug
-                    self.params['taug'] = array(  # type: ignore
-                        self.params['taug'], dtype=float) * float(taug_new)
-        #: update all given params
-        self.params.update({k: kwds[k] for k in kwds if k in self.param_keys})
-        self.params = {k: self.params[k] for k in self.param_keys}  # ! ensure order
-        #: ! Note: Keep params within specified bounds (keep_feasible is handled by Bounds)
-        #: ! Note: that only bounded params are updated
-        self.params = self.update_bounded_params(self.params)
-        if 'tM' in kwds:
-            self.tM = array(kwds['tM'], dtype=float).flatten()
-        if 'N' in kwds and kwds.get('N') is not None:
-            self.t = linspace(0, 24, num=int(kwds['N']))
-        if 'seed' in kwds:
-            self.rng = default_rng(seed=kwds['seed'])
-        if 'eps' in kwds:
-            self.eps = kwds['eps']
+                    self.params["taug"] = array(  # type: ignore
+                        self.params["taug"], dtype=float
+                    ) * float(taug_new)
+        #: update all given params by updating the private dict directly
+        self._params.update(**kwds)
+        #: Important next line:
+        #: ! Keeps params within specified bounds (keep_feasible is handled by Bounds)
+        #: ! Ensures that only bounded params are updated
+        self.params = self.update_bounded_params(self.params)  # type: ignore
+        if "tM" in kwds:
+            # clean up tM (handle string inputs, etc.)
+            # ! caution with this (string injection is possible)
+            tM_raw = kwds["tM"]
+            if isinstance(tM_raw, str):
+                # extract numbers from the string
+                tM = [
+                    float(x)
+                    for x in tM_raw.split(",")
+                    if x.strip().replace(".", "", 1).isdigit()
+                ]
+            elif isinstance(tM_raw, (int, float)):
+                tM = [tM_raw]
+            else:
+                try:
+                    tM = list(tM_raw)
+                except TypeError:
+                    raise TypeError(
+                        f"Invalid type for tM: {type(tM_raw)}. Must be a list, tuple, or string of numeric values."
+                    )
+            try:
+                tM = [
+                    float(x) for x in tM if isinstance(x, (int, float))
+                ]  # type: ignore
+            except ValueError:
+                raise ValueError(
+                    f"Invalid value in tM: {tM}. All values must be numeric."
+                )
+            self.tM = array(tM, dtype=float).flatten()
+        if kwds.get("N") is not None:
+            self.t = linspace(0, 24, num=int(kwds["N"]))
+        if kwds.get("seed") is not None:
+            self.rng = default_rng(seed=kwds["seed"])
+        if kwds.get("eps") is not None:
+            self.eps = kwds["eps"]
         #: check all parameters are present and valid types
         for pkey in self.bounded_param_keys:
-            if pkey not in self.params:
-                raise ValueError(f"Parameter '{pkey}' not found.")
-            if not isinstance(self.params[pkey], (float, int)):
+            if not isinstance(getattr(self.bounded_params, pkey), (float, int)):
                 raise TypeError(f"Parameter '{pkey}' must be numeric.")
 
     @property
     def d(self) -> float:
         """d : float
-             Offset from UTC solar noon for the estimated latitude (hours).
+        Offset from UTC solar noon for the estimated latitude (hours).
         """
-        return self.params["d"]
+        return self.bounded_params.d
 
     @property
     def taup(self) -> float:
         """taup : float
-                Approximate photoperiod (hours).
+        Approximate photoperiod (hours).
         """
-        return self.params["taup"]
+        return self.bounded_params.taup
 
     @property
     def n_meals(self):
@@ -383,28 +477,29 @@ class CMASleepWakeModel:
     @property
     def taug(self) -> ndarray:
         """taug: get an array broadcasted to: (, number_of_meals)."""
-        taug_ = self.params["taug"]
-        taug_vector = broadcast_to(taug_, (self.n_meals, ))
+        taug_ = self.bounded_params.taug
+        taug_vector = broadcast_to(taug_, (self.n_meals,))
         return taug_vector
 
     @property
     def B(self) -> float:
         """Return the current bias parameter value (B)."""
-        return self.params["B"]
+        return self.bounded_params.B
 
     @property
     def Cm(self) -> float:
         """return the current Cm param value."""
-        return self.params["Cm"]
+        return self.bounded_params.Cm
 
     @property
     def toff(self) -> float:
-        return self.params["toff"]
+        return self.bounded_params.toff
 
     def E_L(self, t=None):
         if t is None:
-            t = self.t
-        return Light(0.025 * power((t - 12.0 - self.d), 2) / (self.eps + self.taup))  # type: ignore
+            t = array(self.t)
+        # type: ignore
+        return Light(0.025 * power((t - 12.0 - self.d), 2) / (self.eps + self.taup))
 
     @property
     def L(self):
@@ -414,10 +509,14 @@ class CMASleepWakeModel:
         """compute the estimated relative Melatonin signal."""
         if t is None:
             t = self.t
-        m_out = power((1.0 - self.L), 3) * power(cos(-(t - 3.0 - self.d) * pi / 24.0), 2)  # type: ignore
+        m_out = power((1.0 - self.L), 3) * power(
+            cos(-(t - 3.0 - self.d) * pi / 24.0), 2
+        )  # type: ignore
         if self.rng is not None:
             # ! tiny amount of random noise
-            m_out = m_out + self.rng.uniform(low=-self.eps, high=self.eps, size=self.N)  # type: ignore
+            # type: ignore
+            m_out = m_out + \
+                self.rng.uniform(low=-self.eps, high=self.eps, size=self.N)
         return m_out
 
     @property
@@ -426,12 +525,22 @@ class CMASleepWakeModel:
 
     @property
     def c(self):
-        return (4.9 / (1.0 + self.taup)) * pi * E(power((self.L - 0.88), 3)) * \
-            E(0.05 * (8.0 - self.t + self.d)) * E(2.0 * power(-self.m, 3))  # type: ignore
+        return (
+            (4.9 / (1.0 + self.taup))
+            * pi
+            * E(power((self.L - 0.88), 3))
+            * E(0.05 * (8.0 - self.t + self.d))
+            * E(2.0 * power(-self.m, 3))
+        )  # type: ignore
 
     @property
     def a(self):
-        return (E(power((-self.c * self.m), 3)) + exp(-0.025 * power((self.t - 13 - self.d), 2)) * self.E_L(t=0.7 * (27 - self.t + self.d))) / 2.0  # type: ignore
+        # type: ignore
+        return (
+            E(power((-self.c * self.m), 3))
+            + exp(-0.025 * power((self.t - 13 - self.d), 2))
+            * self.E_L(t=0.7 * (27 - self.t + self.d))
+        ) / 2.0
 
     @property
     def I_S(self):
@@ -441,7 +550,12 @@ class CMASleepWakeModel:
     def I_E(self):
         return self.a * self.I_S
 
-    def calc_Gt(self, t: Optional[ndarray | float | int] = None, dt: Optional[float] = None, n: int = 1) -> DataFrame:
+    def calc_Gt(
+        self,
+        t: Optional[ndarray | float | int] = None,
+        dt: Optional[float] = None,
+        n: int = 1,
+    ) -> DataFrame:
         """
         Calculates Gt for given time frame.
 
@@ -458,11 +572,29 @@ class CMASleepWakeModel:
         elif t is None:
             if dt is None:
                 dt = np_abs(self.t[-1] - self.t[-2])
-            t = mod(linspace(
-                self.t[-1] + dt, self.t[-1] + (n - 1) * dt, num=n), 24)  # type: ignore
-        Gt = vectorized_G(t, self.I_E[-1], self.tM, self.taug, self.B, self.Cm, self.toff)  # type: ignore
-        df_gt = DataFrame({'Gt{}'.format(i): Gt[i] for i in range(Gt.shape[0])}, index=t)  # type: ignore
-        df_gt['Gt'] = nansum(Gt, axis=0)
+            t = mod(
+                linspace(
+                    # type: ignore
+                    self.t[-1] + dt,
+                    self.t[-1] + (n - 1) * dt,
+                    num=n,
+                ),
+                24,
+            )
+        Gt = vectorized_G(
+            # type: ignore
+            t,
+            self.I_E[-1],
+            self.tM,
+            self.taug,
+            self.B,
+            self.Cm,
+            self.toff,
+        )
+        df_gt = DataFrame(
+            {"Gt{}".format(i): Gt[i] for i in range(Gt.shape[0])}, index=t
+        )  # type: ignore
+        df_gt["Gt"] = nansum(Gt, axis=0)
         return df_gt
 
     def update_Gt(self, t=None, dt=None, n=1, keep_tvec_size=True):
@@ -514,23 +646,29 @@ class CMASleepWakeModel:
         """
         return self.G
 
-    def integrate_signal(self, signal: Optional[ndarray] = None,
-                         signal_name: Optional[str] = None,
-                         t0: int | float = 0, t1: int | float = 24,
-                         M: int = 3,
-                         t_extra: Optional[Tuple] = None,
-                         tvec: Optional[ndarray] = None) -> float:
+    def integrate_signal(
+        self,
+        signal: Optional[ndarray] = None,
+        signal_name: Optional[str] = None,
+        t0: int | float = 0,
+        t1: int | float = 24,
+        M: int = 3,
+        t_extra: Optional[Tuple] = None,
+        tvec: Optional[ndarray] = None,
+    ) -> float:
         """Integrate the signal between the hours given, assuming M discrete events.
 
-            t_extra specifies any additional range of 'accepted hours' as an inclusive tuple [te0, te1],
-            to be included in the target time period.
+        t_extra specifies any additional range of 'accepted hours' as an inclusive tuple [te0, te1],
+        to be included in the target time period.
         """
         # trunk-ignore(bandit/B101)
-        assert any([(signal is None), (signal_name is None)]
-                   ), "Must provide exactly one of signal or signal_name"
+        assert any(
+            [(signal is None), (signal_name is None)]
+        ), "Must provide exactly one of signal or signal_name"
         # trunk-ignore(bandit/B101)
-        assert any([(signal is not None), (signal_name is not None)]
-                   ), "Must provide exactly one of signal or signal_name"
+        assert any(
+            [(signal is not None), (signal_name is not None)]
+        ), "Must provide exactly one of signal or signal_name"
         if tvec is None:
             tvec = self.t  # type: ignore
         if signal_name is not None:
@@ -546,12 +684,15 @@ class CMASleepWakeModel:
 
     def morning(self, signal: Optional[ndarray] = None, signal_name=None):
         """compute the total morning integrated signal."""
-        return self.integrate_signal(signal=signal, signal_name=signal_name, t0=4, t1=13)
+        return self.integrate_signal(
+            signal=signal, signal_name=signal_name, t0=4, t1=13
+        )
 
     def evening(self, signal: Optional[ndarray] = None, signal_name=None):
         """Compute the total evening integrated signal."""
-        return self.integrate_signal(signal=signal, signal_name=signal_name,
-                                     t0=16, t1=24, t_extra=(0, 3))
+        return self.integrate_signal(
+            signal=signal, signal_name=signal_name, t0=16, t1=24, t_extra=(0, 3)
+        )
 
     @property
     def columns(self) -> Tuple[str]:
@@ -566,15 +707,15 @@ class CMASleepWakeModel:
         Returns:
             tuple[str]: column names for the Dataframe
         """
-        return ('t', 'c', 'm', 'a', 'I_S', 'I_E', 'L', 'G')  # type: ignore
+        return ("t", "c", "m", "a", "I_S", "I_E", "L", "G")  # type: ignore
 
     @property
     def I_morning(self):
-        return self.morning(signal_name='I_S')
+        return self.morning(signal_name="I_S")
 
     @property
     def I_evening(self):
-        return self.evening(signal_name='I_S')
+        return self.evening(signal_name="I_S")
 
     @property
     def g_morning(self):
@@ -596,7 +737,8 @@ class CMASleepWakeModel:
     @property
     def dt(self):
         #: TimedeltaIndex (in hours)
-        return to_timedelta(self.t, unit='h')  # use 'h' to avoid deprecation warning
+        # use 'h' to avoid deprecation warning
+        return to_timedelta(self.t, unit="h")
 
     @property
     def pvec(self):
@@ -637,13 +779,13 @@ class CMASleepWakeModel:
         #: record estimated meal times
         ismeal = []
         for tm in self.tM:
-            time_since_meal = (df['t'] - tm).abs()
+            time_since_meal = (df["t"] - tm).abs()
             if len(time_since_meal) == 0:
                 continue
             ismeal.append(time_since_meal.idxmin())
-        df['is_meal'] = False
+        df["is_meal"] = False
         if len(ismeal) > 0:
-            df.loc[ismeal, 'is_meal'] = True
+            df.loc[ismeal, "is_meal"] = True
         return df
 
 
@@ -656,7 +798,8 @@ class CMAUtils:
 
     @staticmethod
     def get_hour_of_day(
-            hour: Tuple[float | int] | float | int) -> int | str | Tuple[str | int]:
+        hour: Tuple[float | int] | float | int,
+    ) -> int | str | Tuple[str | int]:
         """
         Get the hour of the day based on the given hour value.
 
@@ -684,18 +827,20 @@ class CMAUtils:
         if hour < 0 or hour > 24:
             raise ValueError("The hour must be between 0 and 24.")
         if hour == 0 or hour == 24:
-            return '12AM'
+            return "12AM"
         elif hour == 12:
-            return '12PM'
+            return "12PM"
         elif hour < 12:
-            return f'{int(hour)}AM'
+            return f"{int(hour)}AM"
         else:
-            return f'{int(hour) - 12}PM'
+            return f"{int(hour) - 12}PM"
 
     @staticmethod
-    def label_meals(df: DataFrame,
-                    rounded: Optional[Callable] = round_to_nearest_integer,
-                    as_str: bool = False) -> Tuple[str | int | float]:
+    def label_meals(
+        df: DataFrame,
+        rounded: Optional[Callable] = round_to_nearest_integer,
+        as_str: bool = False,
+    ) -> Tuple[str | int | float]:
         """Label the meal times in a CMA model results dataframe.
         Parameters:
             df (DataFrame): The CMA model results dataframe.
@@ -713,14 +858,14 @@ class CMAUtils:
             (0, 8, 16)
         """
         #: get the meal times
-        mealtimes = df.loc[df['is_meal'], 't']
+        mealtimes = df.loc[df["is_meal"], "t"]
         if rounded is not None:
             mealtimes = mealtimes.apply(rounded)
         tM = mealtimes.to_numpy(dtype=float).tolist()
         if as_str:
             tM = CMAUtils.get_hour_of_day(tM)
         if not isinstance(tM, tuple):
-            if hasattr(tM, '__iter__'):
+            if hasattr(tM, "__iter__"):
                 tM = tuple(tM)  # type: ignore
             else:
                 tM = (tM,)
