@@ -1,11 +1,9 @@
-import pandas as pd
 import pfun_path_helper as pph
-pph.append_path(path=pph.get_lib_path('pfun_cma_model'))
-
+import pandas as pd
 from pfun_cma_model.engine.data_utils import format_data
-
-
-import pytest
+from . import test_base
+test_base.setup_test_environment()
+pph.append_path(path=pph.get_lib_path('pfun_cma_model'))
 
 
 class TestFormatData:
@@ -27,5 +25,5 @@ class TestFormatData:
         # Check that the output has the expected columns
         assert set(output_df.columns) == {'time', 'value', 'tod', 't', 'G'}
         # Check that the output has the expected number of rows
-        assert len(output_df) == 1024  # 1024 samples, see 'pfun_cma_model.engine.data_utils.downsample_data'
-
+        # 1024 samples, see 'pfun_cma_model.engine.data_utils.downsample_data'
+        assert len(output_df) == 1024
